function [xi1, xi2, there_are_2, is_empty, is_different] ... 
    = extended_intersect(x, y1, y2, there_are_2_y)
% RBK December 14, 2005 --
% [xi1, xi2, there_are_2, is_empty, is_different] ...
%    = extended_intersect(x, y1, y2, there_are_2_y)
% Computes the intersection of an interval x with one or two
% (possibly semi-infinite) intervals y1 and y2.  (The idea is
% that y1 and y2 can represent the complement of an interval,
% but they need not do so.)  If there_are_2_y is 0, then
% only the intersection with y1 is computed.  If the intersection
% consists of two intervals, then there_are_two is set to 1, and
% the intervals are stored in xi1 and xi2;  if there is only one
% interval, then there_are_two is set to 0, and the interval is
% stored in xi1;  if the intersection is empty, then is_empty
% is set to 1;  otherwise is_empty is set to 0.
% is_different is set to 1 if the intersection is different from
% x, and is set to 0 otherwise.
% This routine needs INTLAB.
is_empty = 0;
xi1 = [];
xi2 = [];
if (there_are_2_y)
    there_are_2 = 1;
    if(intersects(x,y1))
        xi1 = intersect(x,y1);
    else
        xil = [];
    end
    if (~isempty(xi1))
        is_different = really_in(xi1,x);
        if(~is_different)
            there_are_2 = 0;
            return;
        end
        if(intersects(x,y2))
            xi2 = intersect(x,y2);
        else
            xi2 = [];
        end
        if (isempty(xi2))
            there_are_2 = 0;
        end
    else
        there_are_2 = 0;
        if(intersects(x,y2))
            xi1 = intersect(x,y2);
        else
            xil = [];
        end
        is_empty = isempty(xi1);
        if (is_empty)
            is_different = 1;
        else
            is_different = really_in(xi1,x);
        end
    end        
else
    there_are_2 = 0;
    if(intersects(x,y1))
        xi1 = intersect(x,y1);
    else
        xil = [];
    end
    is_empty = isempty(xi1);
    if (is_empty)
        is_different = 1;
    else
        is_different = really_in(xi1,x);
    end
end

function [it_is] = really_in(x,y);
   epsilon = 1e-2;
   it_is = (inf(x) >= inf(y)+ epsilon*abs(inf(y))) | ...
           (sup(x) <= sup(y)- epsilon*abs(sup(y)));
end

end