function [new_x_k, second_new_x_k, there_are_2,... 
    numerator, denominator]...
    = gauss_seidel_step(k, Y_k, A, b, x)
% [new_x_k, second_new_x_k, there_are_2]...
%  = gauss_seidel_step(k, Y_k, A, b, x) returns
% the result of applying a Gauss--Seidel step with variable k,
% preconditioner matrix Y_k, and initial guess x.  The variable
% there_are_2 is set to 1 if two semi-infinite intervals are returned,
% in which case second_new_x_k has the second interval;  there_are_2
% is set to 0 and second_new_x_k is not set if there is only 
% one interval returned.
% RBK 11/17/2005.
   n = size(A,2);
   Atilde_k = Y_k*A;
   btilde_k = Y_k*b;
   numerator = btilde_k;
   new_x_k = x(k);
   second_new_x_k = x(k);
   for j = 1:k-1
       numerator = numerator - Atilde_k(j)*x(j);
   end
   for j = k+1:n
       numerator = numerator - Atilde_k(j)*x(j);
   end
   denominator = Atilde_k(k);  
   numerator;
   denominator;
   if (~in(0,denominator))
       there_are_2 = 0;
       new_x_k = numerator / denominator;
   elseif (~in(0,numerator))
       there_are_2 = 1;
       supnum = sup(numerator);
       if(supnum < 0) 
           if sup(denominator)==0
               tmp1 = infsup(-Inf,-Inf);
           else
               tmp1 = midrad(supnum,0) / midrad(sup(denominator),0);
           end
           if inf(denominator) == 0
               tmp2 = infsup(Inf,Inf);
           else
               tmp2 = midrad(supnum,0) / midrad(inf(denominator),0);
           end
           new_x_k = infsup(-Inf,sup(tmp1));
           second_new_x_k = infsup(inf(tmp2),inf);
       else
           infnum = inf(numerator);
           if inf(denominator)==0
               tmp1 = infsup(-Inf,-Inf)
           else
               tmp1 = midrad(infnum,0) / midrad(inf(denominator),0);
           end
           if sup(denominator) == 0
               tmp2 = infsup(Inf,Inf)
           else
               tmp2 = midrad(infnum,0) / midrad(sup(denominator),0);
           end
           new_x_k = infsup(-Inf,sup(tmp1));
           second_new_x_k = infsup(inf(tmp2),Inf);
       end
   else
       there_are_2=0;
       new_x_k = infsup(-Inf,Inf);
   end
end
       
       
       
       
