function [A, b, x, s] = random_test_matrix(m, n, B, R, s0, use_s0)
% RBK 2005/12/16 -- this program generates an m by n interval
% matrix, an interval m-vector b, and an interval n-vector x
% as follows:
%
% The elements of A and b have centers that are uniformly
% randomly distributed in the interval [-1,1] and random
% radii that are uniformly distributed in [0,B].
%
% The elements of x are of the form [-r,r], where r is
% uniformly distributed and random in the interval [0,R].
% 
% use_s0 should be set to either 0 or 1.  If 
% use_s0 is set to 1, then the state of rand is set
% to s0 before computations.
%
% On return, 's' is the state of the Matlab random number
% generator prior to beginning the computation.  This
% can be used.

if(use_s0)
    rand('state',s0);
end
    
s = rand('state');

A_center = 2*rand(m,n)-1;
A_radius = B*rand(m,n);
A = midrad(A_center,A_radius);

b_center = 2*rand(m,1) - 1;
b_radius = B*rand(m,1);
b = midrad(b_center,b_radius);

x_radii = R*rand(n,1);
x = infsup(-x_radii,x_radii);

end
